package sun.text.normalizer;

import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStream;

public class IntTrie extends Trie
{
    private int m_initialValue_;
    private int[] m_data_;
    
    public IntTrie(InputStream inputStream, Trie.DataManipulate dataManipulate) throws IOException {
        super(inputStream, dataManipulate);
        if (!this.isIntTrie()) {
            throw new IllegalArgumentException("Data given does not belong to a int trie.");
        }
        return;
    }
    
    public final int getCodePointValue(int n) {
        int codePointOffset = this.getCodePointOffset(n);
        return (codePointOffset >= 0) ? this.m_data_[codePointOffset] : this.m_initialValue_;
    }
    
    public final int getLeadValue(char c) {
        return this.m_data_[this.getLeadOffset(c)];
    }
    
    public final int getTrailValue(int n, char c) {
        if (this.m_dataManipulate_ == null) {
            throw new NullPointerException("The field DataManipulate in this Trie is null");
        }
        int foldingOffset = this.m_dataManipulate_.getFoldingOffset(n);
        if (foldingOffset > 0) {
            return this.m_data_[this.getRawOffset(foldingOffset, (char)(c & 'Ͽ'))];
        }
        return this.m_initialValue_;
    }
    
    @Override
    protected final void unserialize(InputStream in) throws IOException {
        super.unserialize(in);
        this.m_data_ = new int[this.m_dataLength_];
        DataInputStream dataInputStream = new DataInputStream(in);
        for (int i = 0; i < this.m_dataLength_; ++i) {
            this.m_data_[i] = dataInputStream.readInt();
        }
        this.m_initialValue_ = this.m_data_[0];
    }
    
    @Override
    protected final int getSurrogateOffset(char c, char c2) {
        if (this.m_dataManipulate_ == null) {
            throw new NullPointerException("The field DataManipulate in this Trie is null");
        }
        int foldingOffset = this.m_dataManipulate_.getFoldingOffset(this.getLeadValue(c));
        if (foldingOffset > 0) {
            return this.getRawOffset(foldingOffset, (char)(c2 & 'Ͽ'));
        }
        return -1;
    }
    
    @Override
    protected final int getValue(int n) {
        return this.m_data_[n];
    }
    
    @Override
    protected final int getInitialValue() {
        return this.m_initialValue_;
    }
    
    IntTrie(char[] array, int[] data_, int initialValue_, int n, Trie.DataManipulate dataManipulate) {
        super(array, n, dataManipulate);
        this.m_data_ = data_;
        this.m_dataLength_ = this.m_data_.length;
        this.m_initialValue_ = initialValue_;
    }
}
